<?php

//////////
///
///   db.inc.php
///
//////////

require_once('lib/err.inc.php');



$dbconn = false;

$dbdata = parse_ini_file('etc/RCSDB.ini', true);
$dbdata = $dbdata['dbdata'];



/***************\
*   FUNCTIONS   *
\***************/

function db_init()
{
   global $dbconn, $dbdata;

   if($dbconn) {
      return success(true);
   }

   if(!($dbconn = mysqli_connect($dbdata['host'], $dbdata['user'], $dbdata['pass'], $dbdata['db']))) {
      return error(false, false, $GLOBALS['errno']['db'] + mysqli_connect_errno(), mysqli_connect_error());
   }

   if(!mysqli_set_charset($dbconn, 'utf8')) {
      return error(false, false, $GLOBALS['errno']['db'] + mysqli_errno($dbconn), mysqli_error($dbconn));
   }

   return success(true);
}

function db_escape(&$arg)
{
   global $dbconn;

   $ret = db_init();

   if(!is_success($ret)) {
      return $ret;
   }

   switch(gettype($arg)) {
      case 'string':
         $arg = mysqli_real_escape_string($dbconn, $arg);
         break;
      case 'array':
      case 'object':
         foreach($arg as &$a) {
            db_escape($a);
         }
         unset($a);
         break;
   }

   return success(true);
}

function db_query($sql, $err_map = false)
{
   global $dbconn;

   $ret = db_init();

   if(!is_success($ret)) {
      return $ret;
   }

   unset($ret);

   if(!($result = mysqli_query($dbconn, $sql))) {
      if(isset($err_map[mysqli_errno($dbconn)])) {
         return error($err_map[mysqli_errno($dbconn)][0], $err_map[mysqli_errno($dbconn)][1]);
      }
      return error(false, false, $GLOBALS['errno']['db'] + mysqli_errno($dbconn), mysqli_error($dbconn));
   }

   if($result === true) {
      return success(true);
   }

   if(mysqli_num_rows($result) == 0) {
      mysqli_free_result($result);
      return success(false);
   }

   $types = mysqli_fetch_fields($result);

   while($row = mysqli_fetch_object($result)) {
      foreach($types as $t) {
         switch($t->type) {
            case MYSQLI_TYPE_LONG:
               settype($row->{$t->name}, 'integer');
               break;
            case MYSQLI_TYPE_DATETIME:
               xmlrpc_set_type($row->{$t->name}, 'datetime');
               break;
            case MYSQLI_TYPE_BLOB:
            case MYSQLI_TYPE_LONG_BLOB:
               xmlrpc_set_type($row->{$t->name}, 'base64');
               break;
         }
      }
      $ret[] = $row;
   }

   mysqli_free_result($result);

   return success($ret);
}

function db_prepare($sql)
{
   global $dbconn;

   $ret = db_init();

   if(!is_success($ret)) {
      return $ret;
   }

   if(!($ret = mysqli_prepare($dbconn, $sql))) {
      return error(false, false, $GLOBALS['errno']['db'] + mysqli_errno($dbconn), mysqli_error($dbconn));
   }

   return success($ret);
}

function db_execute($stmt, $params)
{
   $argv[0] = $stmt;
   $argv[1] = '';
   $blob = array();
   $argc = 0;
   $null = NULL;

   foreach($params as $p) {
      switch($p['t']) {
         case 's':
            $argv[1] .= 's';
            $argv[] = $p['v'];
            break;
         case 'i':
            $argv[1] .= 'i';
            $argv[] = $p['v'];
            break;
         case 'b':
            $argv[1] .= 'b';
            $blob[] = $argc;
            $argv[] = $null;
            break;
         default:
            return error('db', 'nosuchtype');
            break;
      }

      $argc++;
   }

   if(!($ret = call_user_func_array('mysqli_stmt_bind_param', $argv))) {
      return error('db', 'generic');
   }

   foreach($blob as $b) {
      if(!($fp = fopen($params[$b]['v'], 'rb'))) {
         return error('session', 'nosuchfile');
      }

      while (!feof($fp)) {
         if(!mysqli_stmt_send_long_data($stmt, $b, fread($fp, 10485760))) {
            fclose($fp);
            return error(false, false, $GLOBALS['errno']['db'] + mysqli_stmt_errno($stmt), mysqli_stmt_error($stmt));
         }
      }
      fclose($fp);
   }

   if(!($ret = mysqli_stmt_execute($stmt))) {
      return error(false, false, $GLOBALS['errno']['db'] + mysqli_stmt_errno($stmt), mysqli_stmt_error($stmt));
   }

   return success(true);
}

?>
